/**
 * class Controller
 *   
 * Der Controller ist die Eingabeschnittstelle zwischen Benutzer und Modell.
 * Er interpretiert die empfangenen Eingabedaten und bergibt sie dem Modell.
 * Ein Controller ist meist auf ein spezielles View zugeschnitten. Ist sein 
 * Verhalten vom Model abhngig, so muss er auch auf nderungen im Model
 * reagieren knnen.
 * 
 * @author lray
 * @version 2007-02-16
 */    

  var TextLineControl = Class.create();

  TextLineControl.prototype = {

    oVTnTChatInput: Object(),
    vTnTInputForm: Object(),
    vTnTInputFormInput: Object(),
    oVTnTChatInteraction: Object(),
    oModel: Object(),
    oLogger:Object(),  
    sClassPath: String("textLineControl"),
    sName: String("TextLine-Control"),
    oUser2TalkFor: Object(),

  /**
   * Initialisiert Model und View.
   * @param oMap zum Controller gehoeriger View
   * @param m Mathematisches Modell
   */
    initialize: function( oModel, sName, oUser )
    {
      this.oLogger = new Log4js.getLogger("vtntTest");

      if ( oModel instanceof vTnTModel) {
        this.oModel = oModel;
      } else {
          this.oLogger.warn("Control: entweder Modell oder View nicht bergeben, laufe im Leerlauf!")
      }      
      
      if ( oUser instanceof User) 
        this.oUser2TalkFor = oUser;
      else 
        this.oLogger.warn("TextLineControl.initialize: kein Referenz-User uebergeben")
      
      if (typeof(sName) == 'String') this.sName = sName;
      
      this.generateSource();
      
    },

  /**
   * Setzt Model und View zurck.
  */
    release: function()
    {
      this.oModel = null;
    },

  /* --------------------------------------------------*/
  // handleEvent-Methode
  
    showQuickNote: function() {
      this.oVTnTInputForm.setAttribute("class","visible");
      this.oVTnTInputForm.setAttribute("style","display: block;");
      try { this.oVTnTInputFormInput.focus(); } catch (e) {};
    },
    
    clearMessage: function() {
      this.oModel.setViewMode(0, true);
    },

    ignore: function() {
      this.oModel.setIgnoreTextLines(true);
      this.clearMessage();
      $("vTnTChat").setAttribute("class","minimized and_closed");
    },

    doSetChatline: function(text) {
      var myTextLine = new TextLine();
      myTextLine.setX(this.oUser2TalkFor.getX());
      myTextLine.setY(this.oUser2TalkFor.getY());
      myTextLine.setLoudness(this.oUser2TalkFor.getLoudness());
      myTextLine.setSId(this.oUser2TalkFor.getId()),
      myTextLine.setId(Math.round(Math.random()*1000));
      myTextLine.setText(text.value);
      myTextLine.setTimestamp(new Date().getTime());
      this.oModel.addTextLine(myTextLine, false);
      text.value="";
      if (this.oModel.getTextLines2Show() ==1 )
    	   $("vTnTChat").setAttribute("class","minimized and_closed");
    },
 
  /**
   * Verarbeiten der Tastatureingaben.
   */
    start: function() {
      /*String s = "";
      do
      {
        s = IOTools.readLine();
        if( s.equals( "j")) model.doSomething();
        if( s.equals( "n")) break;
      }
      while( true);
      view.release();*/
    },
    
    paint: function() {
    	return this.oVTnTChatInput;
    },
    
    toString: function() {
      return this.sName;
    },
    
    generateSource: function() {
      this.oVTnTChatInput = document.createElement('div');
      this.oVTnTChatInput.setAttribute('id','vTnTChatInput')
      this.oVTnTInputForm = document.createElement('form');
      this.oVTnTInputForm.setAttribute('id','vTnTInputForm');
      this.oVTnTInputForm.setAttribute('action', "javascript:"+this.sClassPath+".doSetChatline($('vTnTInputFormInput'));");
      this.oVTnTChatInput.appendChild(this.oVTnTInputForm);
      
      this.vTnTInputFormInput = document.createElement('input');
      this.vTnTInputFormInput.setAttribute('type','text');
      this.vTnTInputFormInput.setAttribute('id','vTnTInputFormInput');
      this.vTnTInputFormInput.setAttribute('name','input');
      this.oVTnTInputForm.appendChild(this.vTnTInputFormInput);
      
      var vTnTInputFormSubmit = document.createElement('input');
      vTnTInputFormSubmit.setAttribute('type','submit');
      vTnTInputFormSubmit.setAttribute('value','go');
      this.oVTnTInputForm.appendChild(vTnTInputFormSubmit);
      
      var vTnTChatInteraction = document.createElement('ul');
      vTnTChatInteraction.setAttribute('id','vTnTChatInteraction');
      
      var tmpArray= new Array();
      tmpArray[0] = new Object();
      tmpArray[0]["href"] = "javascript:"+this.sClassPath+".showQuickNote();";
      tmpArray[0]["title"] = "schnellantwort";

      tmpArray[1] = new Object();
      // TODO: Buggy
      tmpArray[1]["href"] = "javascript:control.getControl().doWithUser('maxi');";
      //tmpArray[1]["href"] = "javascript:this.oModel.setViewMode(1);";     
      tmpArray[1]["title"] = "chatMode";
      
      tmpArray[2] = new Object();
      tmpArray[2]["href"] = "javascript:"+this.sClassPath+".clearMessage();";
      tmpArray[2]["title"] = "ignorieren";

      tmpArray[3] = new Object();
      tmpArray[3]["href"] = "javascript:"+this.sClassPath+".ignore();";
      tmpArray[3]["title"] = "deaktivieren";

      for (var i = 0; i < tmpArray.length; i++) {
        var tmpElement = document.createElement('li');
        var tmpElement2 = document.createElement('a');
        var tmpElement3 = document.createTextNode(tmpArray[i]["title"]);
        tmpElement2.appendChild(tmpElement3);
        tmpElement2.setAttribute('href',tmpArray[i]["href"]);
        tmpElement.appendChild(tmpElement2);
        vTnTChatInteraction.appendChild(tmpElement);
      }
      this.oVTnTChatInput.appendChild(vTnTChatInteraction);     
    }
}
