/**
 * class Controller
 *   
 * Der Controller ist die Eingabeschnittstelle zwischen Benutzer und Modell.
 * Er interpretiert die empfangenen Eingabedaten und bergibt sie dem Modell.
 * Ein Controller ist meist auf ein spezielles View zugeschnitten. Ist sein 
 * Verhalten vom Model abhngig, so muss er auch auf nderungen im Model
 * reagieren knnen.
 * 
 * @author lray
 * @version 2007-02-16
 */    

  var vTnTController = Class.create();

  vTnTController.prototype = {

    oModel: Object(),
    oMap: Object(),
    oLogger:Object(),  
    sName: String("Controller"),


    SVGDocument:Object(),
    SVGRoot:Object(),
    myTranslation:Object(),
    myScale:Object(),
    myTmpLoudness:Number(),        
    
    TrueCoords:Object(),
    Translation:Object(),
    TranslationX:Number(),
    TranslationY:Number(),
    GrabPoint:Object(),
    BackDrop:Object(),
    DragTarget:Object(),
    DragType:Number(1),

  /**
   * Initialisiert Model und View.
   * @param oMap zum Controller gehoeriger View
   * @param m Mathematisches Modell
   */
    initialize: function( oModel, oMap )
    {
      this.oLogger = new Log4js.getLogger("vtntTest");

      if (oModel == null || oMap == null) {
          this.oLogger.warn("Control: entweder Modell oder View nicht bergeben, laufe im Leerlauf!")
      }
      
      this.oModel = oModel;
      this.oMap  = oMap;

      // SVG-Stuff
      this.SVGRoot = this.oMap.paint();
      this.TrueCoords =   this.oMap.getSVGPoint();
      this.GrabPoint =    this.oMap.getSVGPoint();
      this.BackDrop =     this.oMap.paint();  
      this.myTranslation= this.oMap.getSVGPoint();

//      this.TrueCoords = this.SVGRoot.createSVGPoint();
//      this.GrabPoint = this.SVGRoot.createSVGPoint();
      // this will serve as the canvas over which items are dragged.
      //    having the drag events occur on the mousemove over a backdrop
      //    (instead of the dragged element) prevents the dragged element
      //    from being inadvertantly dropped when the mouse is moved rapidly
      this.BackDrop = document.getElementById('BackDrop');

      
    },

  /**
   * Setzt Model und View zurck.
   */
    release: function()
    {
      this.oModel = null;
      this.oMap = null;
    },

    toString: function()
    {
      return this.sName;
    },
  /* --------------------------------------------------*/
  // handleEvent-Methode
  
        init:function(evt)
        {
           logger.debug("SVG-Javascript inited");
           // SVGDocument = document.getElementById('vTnTMap').ownerDocument;
      		 // SVGRoot = evt.target;
      		 // SVGRoot = oVMap.paint();
           // these svg points hold x and y values...
           //    very handy, but they do not display on the screen (just so you know)
      },

      over:function(evt)
      {
         //logger.debug("mouse sended 'over'");
         var targetElement = evt.target;
         if (targetElement.getAttribute('class') == 'ava_body'
             && targetElement.parentNode.getAttribute('class') == 'avatar')

            this.oMap.getView().detect( function(value) {
                return value.getId() == targetElement.parentNode.id;
            })
            .showUserData();
      },
      
      out:function(evt)
      {
         var targetElement = evt.target;
         //logger.debug("mouse sended 'out'");
         if (targetElement.getAttribute('class') == 'ava_body'
             && targetElement.parentNode.getAttribute('class') == 'avatar')

            this.oMap.getView().detect( function(value) {
                return value.getId() == targetElement.parentNode.id;
            }).hideUserData();

      },

      grab:function(evt)
      {
         // find out which element we moused down on
         var targetElement = evt.target;
/*         SVGDocument.getElementById("mouseups").firstChild.data = "grabbed.";*/
          // der angewhlte Punkt ist der eigene Massepunkt
    		  if ((targetElement.getAttribute('class') == 'ava_body'
              &&targetElement.parentNode.getAttribute('class')
              &&targetElement.parentNode.getAttribute('class').match(/moveable/) ))
          { 
            targetElement = this.oMap.getView().detect( function(value) {
                return value.getId() == targetElement.parentNode.id;
            });

            this.oLogger.debug("grabbing "+evt.target.id+" for moving and got "+targetElement);    		  	
            targetElement.setMovingType(1);
            this.DragType = 1;
    		  } else {
            // der gewhlte Punkt ist der eigene Umkreis 
            if (targetElement.getAttribute('class') == 'ava_loudness'
              &&targetElement.parentNode.getAttribute('class')
              &&targetElement.parentNode.getAttribute('class').match(/moveable/) ) {
    
          		  	targetElement = this.oMap.getView().detect( function(value) {
                    return value.getId() == targetElement.parentNode.id;
                });
                targetElement.setMovingType(2);
    		  	    this.DragType = 2;
                this.oLogger.debug("grabbing "+evt.target.id+" for resizing.");                
              }
              else
                if (targetElement == this.SVGRoot) {
                
                }
                else targetElement = null;
          }
          
    		  //if ( targetElement == SVGRoot)
    		 	//  targetElement = null;

         if (targetElement == this.SVGRoot)
         {
            this.GrabPoint.x = this.TrueCoords.x;
            this.GrabPoint.y = this.TrueCoords.y; 
            this.oLogger.debug("grabbed Background");
            this.DragTarget = targetElement;       
         }
         
         // you cannot drag the background itself, so ignore any attempts to mouse down on it
         if ( targetElement != null && this.BackDrop != targetElement )
         {
            //set the item moused down on as the element to be dragged
            this.DragTarget = targetElement;
            this.DragTarget.setMovingType(this.DragType);

            // move this element to the "top" of the display, so it is (almost)
            //    always over other elements (exception: in this case, elements that are
            //    "in the folder" (children of the folder group) with only maintain
            //    hierarchy within that group
//            DragTarget.parentNode.appendChild( DragTarget );

            // turn off all pointer events to the dragged element, this does 2 things:
            //    1) allows us to drag text elements without selecting the text
            //    2) allows us to find out where the dragged element is dropped (see Drop)

            // we need to find the current position and translation of the grabbed element,
            //    so that we only apply the differential between the current location
            //    and the new location
            //var transMatrix = DragTarget.getCTM();
            //GrabPoint.x = TrueCoords.x - Number(transMatrix.e);
            //GrabPoint.y = TrueCoords.y - Number(transMatrix.f);
            this.GrabPoint.x = this.TrueCoords.x - this.DragTarget.getX();
            this.GrabPoint.y = this.TrueCoords.y - this.DragTarget.getY();
            this.myTmpLoudness = this.DragTarget.getLoudness();
         }
      },


      drag:function(evt)
      {
         // account for zooming and panning
         this.getTrueCoords(evt);
         // if we don't currently have an element in tow, don't do anything
         // wir haben den Massepunkt
         if (this.DragTarget == this.SVGRoot) {
              var newX = this.TrueCoords.x - this.GrabPoint.x;
              var newY = this.TrueCoords.y - this.GrabPoint.y;
              this.oMap.setTranslationX(Number(-1*newX));
              this.oMap.setTranslationY(Number(-1*newY));          
              this.oLogger.debug("visualMap.setTranslation:"+newY+","+newX);
         }
         else {
            // wir haben den Body erwischt
           if ((this.DragTarget instanceof Avatar)&&this.DragTarget.getMovingType() == 1)
           {
              // account for the offset between the element's origin and the
              //    exact place we grabbed it... this way, the drag will look more natural
              var newX = this.TrueCoords.x - this.GrabPoint.x;
              var newY = this.TrueCoords.y - this.GrabPoint.y;
              var translation = this.SVGRoot.currentTranslate;
  
              // apply a new tranform translation to the dragged element, to display
              //    it in its new location
              this.DragTarget.setX(newX);
              this.DragTarget.setY(newY);
           }
  
            // wir haben den Umkreis
           if ((this.DragTarget instanceof Avatar)&&this.DragTarget.getMovingType() ==2)
           {
              // account for the offset between the element's origin and the
              //    exact place we grabbed it... this way, the drag will look more natural
              var newX = this.TrueCoords.x - this.GrabPoint.x - this.DragTarget.getX();
              var newY = this.TrueCoords.y - this.GrabPoint.y - this.DragTarget.getY();
              var newLoudness = Math.round( Math.sqrt( Math.pow(newX,2) + Math.pow(newY,2)));  
              // apply a new tranform translation to the dragged element, to display
              //    it in its new location
              this.oLogger.debug("Radius-Anpassung X:"+newX+" Y:"+newY+" ergibt "+newLoudness);              
              this.DragTarget.setLoudness(Number( this.myTmpLoudness + newLoudness ) );
           }
        }
      },


      drop:function(evt)
      {
         // if we aren't currently dragging an element, don't do anything
         if ( this.DragTarget )
         {
            // since the element currently being dragged has its pointer-events turned off,
            //    we are afforded the opportunity to find out the element it's being dropped on
            var targetElement = evt.target;
          
            // bewege Hintergrund
            if (this.DragTarget == this.SVGRoot) {
                this.DragTarget = null;
//                myTranslation = TrueCoords;
            }
            
            //genderter Radius
            if ( ( this.DragTarget instanceof Avatar)&&this.DragTarget.getMovingType() == 2 )
            {
/*               DragTarget.setAttributeNS(null, 'pointer-events', 'visibleStroke');*/
  
  /*          	 var aktTransform = DragTarget.parentNode.getAttributeNS(null,'transform');*/ 
  
               this.oLogger.debug(this.DragTarget.getId() + ' has now a changed Radius ' + targetElement.id 
               +' and has the following XY '+this.DragTarget.getX()+" "+this.DragTarget.getY());
              
                this.DragTarget.setMovingType(0);
                var oTmpModel = this.oModel.getUserById(this.DragTarget.getId());
                oTmpModel..setLoudness(this.DragTarget.getLoudness());
                this.oModel.updateUser(oTmpModel);
                
                this.DragTarget = null;
            }
            else
            if ( ( this.DragTarget instanceof Avatar) && this.DragTarget.getMovingType() == 1 ){
                // turn the pointer-events back on, so we can grab this item later
               // for this example, you cannot drag an item out of the folder once it's in there;
               //    however, you could just as easily do so here

               this.oLogger.debug(this.DragTarget.getId() + ' has been dropped on top of ' + targetElement.id 
               +' and has the following XY '+this.DragTarget.getX()+","+this.DragTarget.getY());

              
              var oTmpModel = this.oModel.getUserById(this.DragTarget.getId());
              
              oTmpModel.setX(this.DragTarget.getX());
              oTmpModel.setY(this.DragTarget.getY());

              this.oModel.updateUser(oTmpModel);            
              this.DragTarget.setMovingType(0);
              this.DragTarget = null;
            } else
                this.DragTarget = null;
            // set the global variable to null, so nothing will be dragged until we
            //    grab the next element
         }
      },


      getTrueCoords:function (evt)
      {
         // find the current zoom level and pan setting, and adjust the reported
         //    mouse position accordingly

         var newScale = this.SVGRoot.currentScale;
         var translation = this.SVGRoot.currentTranslate;
         if ((this.DragTarget != this.SVGRoot)) {
           this.TrueCoords.x = ((evt.clientX - translation.x)/newScale);
           this.TrueCoords.y = ((evt.clientY - translation.y)/newScale);
         } else {
           this.TrueCoords.x = (evt.clientX);
           this.TrueCoords.y = (evt.clientY);
         }
         
         //logger.debug("Scale "+newScale+" Translation "+translation.x+","+translation.y+" with TrueCords "+TrueCoords.x+","+TrueCoords.y);
/*         SVGDocument.getElementById("mouseups").firstChild.data = "Cords= " + TrueCoords.x + " / " + TrueCoords.y;*/
      },
  
  /**
   * Verarbeiten der Tastatureingaben.
   */
    start: function() {
      /*String s = "";
      do
      {
        s = IOTools.readLine();
        if( s.equals( "j")) model.doSomething();
        if( s.equals( "n")) break;
      }
      while( true);
      view.release();*/
    },
    
    getName: function() {
      return this.sName;
    },

    //Beginn Paste
      doWithUser:function(what2do) {
        switch (what2do) {
          case "mini":  
                        this.oModel.setViewMode(0);
                        var vTnTName = "vTnT";
                        var vTnT =  document.getElementById(vTnTName);
                        var vTnTId = $("vTnTMapControl");
                        var vTnTBG = document.getElementById("vTnTBackground");
                        
                        vTnTId.style.display = "none";
                        //$('vTnTChat').style.display = "none";
                        
                        vTnT.style.zIndex = 10000;
                        vTnT.style.width = "640px";
                        vTnT.style.position = "static";
                        
                        
                        document.getElementById("vTnTVisual").style.width = "80px";
                        document.getElementById("vTnTVisual").style.height = "240px";
                        try {new Effect.Fade('vTnTBackground', { duration: 0.6, from: 0.8, to: 0.0 });} catch (e) {};

                        var tmpMyOwn = this.oModel.getOwnUser();
                        
                        tmpMyOwn.each(function(item) {item.setActive(false);});
                        
                        $('vTnTChatInteraction').style.display = "block";
                        $("vTnTChat").setAttribute("class","minimized");
                        
                        // Hiermit wird ein anderer View (TextLineChat) beeinflut.
                        // wie bekommen wir das sauber hin?
                        /*this.aTLCol.setLines2Show(1);
                        this.aTLCol.setTextLength(35);
                        this.aTLCol.setNameFilter(new Array("ID1"));
                        this.aTLCol.setTimestamp(new Date((new Date().getTime())-10000));
    
                        if (this.aTLCol.updateView() <= 0) {
                          $("vTnTChat").setAttribute("class","minimized and_closed");
                        }
                        */

                        break;   
    
          case "maxi":  
                        this.oModel.setViewMode(1);
                        var pageSize = getPageSize();
                        var vTnTName = "vTnT";
                        var vTnTBG = $("vTnTBackground");
                        var vTnT =  $(vTnTName);
                        var vTnTId = $("vTnTMapControl");
                        
                        vTnT.style.zIndex = 10000;
                        vTnT.style.display = "none";
                        vTnT.style.width = "640px";

                        //vTnT.style.height = "480px";                        
                        vTnT.style.backgroundColor = "#ffffff";
                        vTnT.style.position = "absolute";
                        vTnT.style.padding = "20px 20px 20px 20px";

                        vTnTBG.style.width  = pageSize[0]+"px";
                        vTnTBG.style.height = pageSize[1]+"px";

                        vTnTId.style.display = "block";
                        $('vTnTChat').setAttribute("class","");
                        $('vTnTChatInteraction').style.display = "none";                 
                        document.getElementById("vTnTVisual").style.width = "600px";
                        document.getElementById("vTnTVisual").style.height = "440px";

                        new Effect.Appear('vTnTBackground', { duration: 0.6, from: 0.0, to: 0.8 });
                        vTnT.style.width = 660+"px";
                        vTnT.style.left = Number((pageSize[2] / 2)-320)+"px";        
                        vTnT.style.top = Number((pageSize[3] / 2)-300)+"px";
                        new Effect.Appear(vTnTName, { duration: 0.6, from: 0.0, to: 1.0 });    

                        var tmpMyOwn = this.oModel.getOwnUser();
                        tmpMyOwn.each(function(item) {item.setActive(true);});
                        
                        // This modifies a different View (TextLineChat)
                        // How to handle this line?
                        /*this.aTLCol.setLines2Show(150);
                        this.aTLCol.setTextLength(-1);
                        this.aTLCol.setNameFilter(new Array()); */
    
                        $("vTnTChat").removeAttribute("class");
                        break;   
      
          default:
            this.oLogger.warn("doWithUser: unbekannte Aufforderung:"+what2do);
            break;
        }    
  },    

}
